@@ -1,8 +1,8 @@
 import logging
 
 from ..utils import (
-    _get_juju_home, path_join, yaml_load, ErrorExit, yaml_dump, temp_file,
-    _check_call)
+    _get_juju_home, path_join, yaml_load, ErrorExit,
+    yaml_dump, temp_file, _check_call)
 
 
 class BaseEnvironment(object):
@@ -111,8 +111,8 @@ class BaseEnvironment(object):
         delete the machine (ie units have finished executing stop hooks and are
         removed)
         """
-        if (isinstance(mid, int) and mid == 0) or (
-            mid.isdigit() and int(mid) == 0):
+        if ((isinstance(mid, int) and mid == 0) or
+           (mid.isdigit() and int(mid) == 0)):
             # Don't kill state server
             raise RuntimeError("Can't terminate machine 0")
         params = self._named_env(["juju", "terminate-machine"])
@@ -130,11 +130,19 @@ class BaseEnvironment(object):
         if svc_name not in status['services']:
             self.log.warning("Service %s does not exist", svc_name)
             return None
-        units = status['services'][svc_name].get('units', {})
+        svc = status['services'][svc_name]
+        if 'subordinate-to' in svc:
+            ps = svc['subordinate-to'][0]
+            self.log.info(
+                'Service %s is a subordinate to %s, finding principle service'
+                % (svc_name, ps))
+            return self.get_service_address(svc['subordinate-to'][0])
+
+        units = svc.get('units', {})
         unit_keys = list(sorted(units.keys()))
         if unit_keys:
             return units[unit_keys[0]].get('public-address', '')
-        self.log.warning("Service %s has no units")
+        self.log.warning("Service %s has no units" % svc_name)
 
     def get_cli_status(self):
         params = self._named_env(["juju", "status"])
@@ -144,3 +152,6 @@ class BaseEnvironment(object):
                 stderr=fh)
         status = yaml_load(output)
         return status
+
+    def add_unit(self, service_name, machine_spec):
+        raise NotImplementedError()
