@@ -4,6 +4,9 @@ class Service(object):
         self.svc_data = svc_data
         self.name = name
 
+    def __repr__(self):
+        return "<Service %s>" % (self.name)
+
     @property
     def config(self):
         return self.svc_data.get('options', None)
@@ -17,9 +20,14 @@ class Service(object):
         return int(self.svc_data.get('num_units', 1))
 
     @property
-    def force_machine(self):
-        return self.svc_data.get('to') or self.svc_data.get(
-            'force-machine')
+    def unit_placement(self):
+        # Separate checks to support machine 0 placement.
+        value = self.svc_data.get('to')
+        if value is None:
+            value = self.svc_data.get('force-machine')
+        if value is not None and not isinstance(value, list):
+            value = [value]
+        return value or []
 
     @property
     def expose(self):
